use anyhow::Result;
use async_graphql::{InputObject, SimpleObject};
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};
use ulid::Ulid;

use crate::scalar::Id;

#[derive(SimpleObject, Debug, FromRow, Deserialize, Serialize, sqlx::Type)]
pub struct Hersteller {
    /// Die Datenbank-ID
    pub id: Id,

    /// Der Name eines Herstellers
    herstellername: String,
}

#[derive(InputObject, Debug)]
pub struct HerstellerCreateInput {
    /// Der Name eines Herstellers
    pub herstellername: String,
}

#[derive(InputObject, Debug)]
pub struct HerstellerUpdateInput {
    /// Die Datenbank-ID
    pub id: Id,

    /// Der Name eines Herstellers
    pub herstellername: Option<String>,
}

impl Hersteller {
    pub async fn read_one(pool: &PgPool, id: &Id) -> Result<Hersteller> {
        let row = sqlx::query_as!(Hersteller, "SELECT * FROM hersteller WHERE id = $1", id)
            .fetch_one(pool)
            .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Hersteller>> {
        let rows = sqlx::query_as!(Hersteller, "SELECT id, herstellername FROM hersteller")
            .fetch_all(pool)
            .await?;

        Ok(rows)
    }

    pub async fn create(pool: &PgPool, input: &HerstellerCreateInput) -> Result<Hersteller> {
        const QUERY: &str = "INSERT INTO hersteller (id, herstellername) VALUES ($1, $2) RETURNING id, herstellername";

        let result: Hersteller = sqlx::query_as::<_, Hersteller>(QUERY)
            .bind::<Id>(Ulid::new().into())
            .bind(&input.herstellername)
            .fetch_one(pool)
            .await?;

        Ok(result)
    }

    pub async fn update(pool: &PgPool, input: &HerstellerUpdateInput) -> Result<Hersteller> {
        sqlx::query!(
            "UPDATE hersteller SET herstellername=$1 WHERE id = $2",
            input.herstellername,
            input.id
        )
        .execute(pool)
        .await?;

        let t = Hersteller::read_one(pool, &input.id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &Id) -> Result<bool> {
        let result = sqlx::query!("DELETE FROM hersteller WHERE id = $1", id)
            .execute(pool)
            .await?;

        Ok(result.rows_affected() > 0)
    }
}
