use anyhow::Result;
use async_graphql::{InputObject, SimpleObject};
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};
use ulid::Ulid;
use uuid::Uuid;

use crate::scalar::Id;

use super::{hersteller::Hersteller, typ::Typ};

#[derive(SimpleObject, FromRow, Deserialize, Serialize, Debug)]
pub struct Modell {
    /// Die Datenbank-ID
    pub id: Id,

    /// Der Geräte-Typ z.B. Monitor
    typ: Typ,

    /// Der Geräte-Hersteller z.B. Dell
    hersteller: Hersteller,

    /// Der Name eines Modells
    modellname: String,
}

#[derive(InputObject, Debug)]
pub struct ModellCreateInput {
    /// Der Name eines Modells
    pub modellname: String,

    /// Die Id eines  Geräte-Typs z.B. Monitor
    pub typ_id: Id,

    /// Die Id eines  Geräte-Herstellers z.B. Dell
    pub hersteller_id: Id,
}

#[derive(InputObject, Debug)]
pub struct ModellUpdateInput {
    pub id: Id,
    pub modellname: Option<String>,
    pub typ_id: Option<Id>,
    pub hersteller_id: Option<Id>,
}

impl Modell {
    pub async fn read_one(pool: &PgPool, id: &Id) -> Result<Modell> {
        let row = sqlx::query_as!(
            Modell,
            r#"
                SELECT
                    m.id,
                    m.modellname,
                    (t.id, t.typname) AS "typ!: Typ",
                    (h.id, h.herstellername) AS "hersteller!: Hersteller"
                FROM modelle AS m
                JOIN typen AS t ON t.id = m.typ_id
                JOIN hersteller AS h ON h.id = m.hersteller_id
                WHERE m.id = $1;
            "#,
            id
        )
        .fetch_one(pool)
        .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Modell>> {
        let row = sqlx::query_as!(
            Modell,
            r#"
                SELECT
                    m.id,
                    m.modellname,
                    (t.id, t.typname) AS "typ!: Typ",
                    (h.id, h.herstellername) AS "hersteller!: Hersteller"
                FROM modelle AS m
                LEFT JOIN typen AS t ON t.id = m.typ_id
                LEFT JOIN hersteller AS h ON h.id = m.hersteller_id
            "#
        )
        .fetch_all(pool)
        .await?;

        Ok(row)
    }

    pub async fn create(pool: &PgPool, input: &ModellCreateInput) -> Result<Modell> {
        const QUERY: &str = "INSERT INTO modelle (id, typ_id, hersteller_id, modellname) VALUES ($1, $2, $3, $4) RETURNING id";

        let row: (Uuid,) = sqlx::query_as(QUERY)
            .bind::<Id>(Ulid::new().into())
            .bind(input.typ_id)
            .bind(input.hersteller_id)
            .bind(&input.modellname)
            .fetch_one(pool)
            .await?;

        let modell = Modell::read_one(pool, &row.0).await?;

        Ok(modell)
    }

    pub async fn update(pool: &PgPool, input: &ModellUpdateInput) -> Result<Modell> {
        sqlx::query!(
            "UPDATE modelle SET modellname=COALESCE($1, modellname), typ_id=COALESCE($2, typ_id), hersteller_id=COALESCE($3, hersteller_id) WHERE id = $4",
            input.modellname,
            input.typ_id,
            input.hersteller_id,
            input.id
        )
        .execute(pool)
        .await?;

        let t = Modell::read_one(pool, &input.id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &Id) -> Result<bool> {
        let result = sqlx::query!("DELETE FROM modelle WHERE id = $1", id)
            .execute(pool)
            .await?;

        Ok(result.rows_affected() > 0)
    }
}
