use anyhow::Result;
use async_graphql::{InputObject, SimpleObject};
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool, Type};
use ulid::Ulid;

use crate::scalar::Id;

#[derive(SimpleObject, Debug, FromRow, Deserialize, Serialize, Type)]
pub struct Typ {
    /// Die ID eines Geräte-Typs
    pub id: Id,

    /// Der Name eines Geräte-Typs
    pub typname: String,
}

#[derive(InputObject, Debug)]
pub struct TypCreateInput {
    pub typname: String,
}

#[derive(InputObject, Debug)]
pub struct TypUpdateInput {
    pub id: Id,
    pub typname: Option<String>,
}

impl Typ {
    pub async fn read_one(pool: &PgPool, id: &Id) -> Result<Typ> {
        const QUERY: &str = "SELECT * FROM typen WHERE id = $1";
        let row: Typ = sqlx::query_as(QUERY).bind(id).fetch_one(pool).await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Typ>> {
        let rows = sqlx::query_as!(Typ, "SELECT id, typname FROM typen")
            .fetch_all(pool)
            .await?;

        Ok(rows)
    }

    pub async fn create(pool: &PgPool, input: &TypCreateInput) -> Result<Typ> {
        const QUERY: &str = "INSERT INTO typen (id, typname) VALUES ($1, $2) RETURNING id, typname";

        let result: Typ = sqlx::query_as::<_, Typ>(QUERY)
            .bind::<Id>(Ulid::new().into())
            .bind(&input.typname)
            .fetch_one(pool)
            .await?;

        Ok(result)
    }

    pub async fn create_many(pool: &PgPool, input: &[TypCreateInput]) -> Result<Vec<Typ>> {
        let mut v1: Vec<&str> = Vec::new();
        input.iter().for_each(|typ| v1.push(&typ.typname));

        let row = sqlx::query_as!(
            Typ,
            r#"
            INSERT INTO typen(typname)
            SELECT * FROM UNNEST($1::text[])
            RETURNING id, typname"#,
            &v1 as _
        )
        .fetch_all(pool)
        .await?;

        Ok(row)
    }

    pub async fn update(pool: &PgPool, input: &TypUpdateInput) -> Result<Typ> {
        sqlx::query!(
            "UPDATE typen SET typname=$1 WHERE id = $2",
            input.typname,
            input.id
        )
        .execute(pool)
        .await?;

        let t = Typ::read_one(pool, &input.id).await?;
        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &Id) -> Result<bool> {
        let result = sqlx::query!("DELETE FROM typen WHERE id = $1", id)
            .execute(pool)
            .await?;

        Ok(result.rows_affected() > 0)
    }
}
